﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WyszukiwarkaTwitter
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(MainPage_Loaded);
            Application.Current.InstallStateChanged += new EventHandler(OnInstallStateChanged);
        }

        // After the Frame navigates, ensure the HyperlinkButton representing the current page is selected
        private void ContentFrame_Navigated(object sender, NavigationEventArgs e)
        {
            foreach (UIElement child in LinksStackPanel.Children)
            {
                HyperlinkButton hb = child as HyperlinkButton;
                if (hb != null && hb.NavigateUri != null)
                {
                    if (hb.NavigateUri.ToString().Equals(e.Uri.ToString()))
                    {
                        VisualStateManager.GoToState(hb, "ActiveLink", true);
                    }
                    else
                    {
                        VisualStateManager.GoToState(hb, "InactiveLink", true);
                    }
                }
            }
        }

        // If an error occurs during navigation, show an error window
        private void ContentFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            e.Handled = true;
            ChildWindow errorWin = new ErrorWindow(e.Uri);
            errorWin.Show();
        }

        private void InstallOffline(object sender, RoutedEventArgs e)
        {
            if (Application.Current.Install())
            {
                InstallLink.Visibility = Visibility.Collapsed;
            }
        }

        void OnInstallStateChanged(object sender, EventArgs e)
        {
            switch (Application.Current.InstallState)
            {
                case InstallState.InstallFailed:
                    break;
                case InstallState.Installed:
                    ToggleInstallLinks(true);
                    break;
                case InstallState.Installing:
                    break;
                case InstallState.NotInstalled:
                    ToggleInstallLinks(false);
                    break;
                default:
                    break;
            }
        }

        void ToggleInstallLinks(bool hidden)
        {
            InstallLink.Visibility = hidden ? Visibility.Collapsed: Visibility.Visible;
            Divider3.Visibility = hidden ? Visibility.Collapsed : Visibility.Visible;
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            if (App.Current.IsRunningOutOfBrowser)
            {
                ToggleInstallLinks(true);
            }
            else
            {
                if (App.Current.InstallState == InstallState.Installed)
                {
                    ToggleInstallLinks(true);
                }
            }
        }

    }
}