﻿using System;
using System.Net;
using System.Net.Browser;
using System.Windows;
using System.Windows.Data;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Navigation;
using System.Windows.Threading;
using System.Windows.Media.Imaging;
using System.Collections.ObjectModel;
using System.Xml;
using System.ServiceModel.Syndication;

using WyszukiwarkaTwitter.Model;
using System.Net.NetworkInformation;

namespace WyszukiwarkaTwitter.Views
{
    public partial class Search : Page
    {
        const string SEARCH_URI = "http://search.twitter.com/search.atom?q={0}&since_id={1}";
        private string _lastId = "0";
        private bool _gotLatest = false;
        ObservableCollection<TwitterSearchResult> searchResults = new ObservableCollection<TwitterSearchResult>();
        PagedCollectionView pcv;
        DispatcherTimer _timer;


        public Search()
        {
            InitializeComponent();

            double interval = 30.0;

            _timer = new DispatcherTimer();
            #if DEBUG
            interval = 10.0;
            #endif
            _timer.Interval = TimeSpan.FromSeconds(interval);
            _timer.Tick += new EventHandler(OnTimerTick);

            pcv = new PagedCollectionView(searchResults);
            pcv.SortDescriptions.Add(new System.ComponentModel.SortDescription("PublishDate", System.ComponentModel.ListSortDirection.Ascending));

            Loaded += new RoutedEventHandler(Search_Loaded);
            NetworkChange.NetworkAddressChanged += new NetworkAddressChangedEventHandler(OnNetworkChanged);
        }

        void OnTimerTick(object sender, EventArgs e)
        {
             SzukajTweetaEx();
        }

        void Search_Loaded(object sender, RoutedEventArgs e)
        {
            SearchResults.ItemsSource = pcv; //połacz z kontrolką DataGrid
            _timer.Start(); // włącz stoper
            SzukajTweetaEx(); // wykonaj pierwsze przeszukanie
            SearchTerm.ItemsSource = Helper.GetSearchTermHistory(); // zapisz historie wyszukiwan
        }

        // Executes when the user navigates to this page.
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            if (this.NavigationContext.QueryString.ContainsKey("term"))
            {
                SearchTerm.Text = this.NavigationContext.QueryString["term"].ToString();
                SzukajTweetaEx();
            }
        }

        private void SzukajTweeta(object sender, RoutedEventArgs e)
        {
            SzukajTweetaEx();
        }

        private void SzukajTweetaEx()
        {
            if (!string.IsNullOrEmpty(SearchTerm.Text)) //sprawdź czy wprowadzono fraze
            {
                _timer.Stop();  //zatrzymaj stoper gdy wyszukujemy dłużej niż zdefiniowany interwał
                BusyIndicator.IsBusy = true; //włącz pasek postępu

                _lastId = Helper.GetLatestTweetId(SearchTerm.Text); // pobierz zapisany identyfikator ostatniego wpisu
                Helper.SaveLatestTweetId(SearchTerm.Text, _lastId); // zapisz historie, nawet gdy brak nowych wpisów

                //przeszukujemy Twitter-a i obsługujemy wyniki wyszukiwania
                WebClient proxy = new WebClient();
                proxy.OpenReadCompleted += new OpenReadCompletedEventHandler(OnReadCompleted);
                proxy.OpenReadAsync(new Uri(string.Format(SEARCH_URI, HttpUtility.UrlEncode(SearchTerm.Text), _lastId)));
            }
        }

        void OnReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                _gotLatest = false; //wyzeruj flage ostatniego wyniku
                XmlReader rdr = XmlReader.Create(e.Result);  //załaduj wyniki do czytnika XML

                SyndicationFeed feed = SyndicationFeed.Load(rdr);  //wypełnij złożone dane (Atom)

                //załaduj każdy wynik do naszej kolekcji
                foreach (var item in feed.Items)
                {
                    searchResults.Add(new TwitterSearchResult() { Author = item.Authors[0].Name, ID = GetTweetId(item.Id), Tweet = item.Title.Text, PublishDate = item.PublishDate.DateTime.ToLocalTime(), Avatar = new BitmapImage(item.Links[1].Uri) });
                    _gotLatest = true; //ustaw flage informującą, że mamy Id ostatniego wyniku
                }

                rdr.Close(); //zamknij czytnik
                Helper.SaveLatestTweetId(SearchTerm.Text, _lastId); //zapisz identyfikator ostatniego wpisu
            }
            else
            {
                //zainicjuj komunikat o błędzie
                ChildWindow errorWindow = new ErrorWindow(e.Error);
                errorWindow.Show();
            }
            BusyIndicator.IsBusy = false; //aktywuj UI
            _timer.Start(); //wyzeruj stoper
        }

        private string GetTweetId(string twitterId)
        {
            string[] parts = twitterId.Split(":".ToCharArray());
            if (!_gotLatest)
            {
                _lastId = parts[2].ToString();
            }
            return parts[2].ToString();
        }

        void OnNetworkChanged(object sender, EventArgs e)
        {
            if (!NetworkInterface.GetIsNetworkAvailable())
            {
                // sieć może być niedostępna, zatrzymaj wyszukiwanie
                SearchButton.IsEnabled = false;
                SearchButton.Content = "BRAK SIECI";
                _timer.Stop();
            }
            else
            {
                SearchButton.Content = "Szukaj";
                SearchButton.IsEnabled = true;
                _timer.Start();
            }
        }

    }
}